@@ -13,12 +13,14 @@ log_file = "log.log"
 traceback = False
 
 
+if "REDIRECT_QUERY_STRING" in os.environ and not os.environ["QUERY_STRING"]:
+    os.environ["QUERY_STRING"] = os.environ["REDIRECT_QUERY_STRING"]
 _SERVER = dict(os.environ)
-arguments = FieldStorage()
+arguments = FieldStorage(environ=os.environ)
 arguments = {k: [_.value for _ in arguments[k]] if isinstance(arguments[k], list) else arguments[k].value for k in arguments}
 _GET = arguments
 _POST = arguments
-_SESSION = SimpleCookie(_SERVER["HTTP_COOKIE"]) if "HTTP_COOKIE" in _SERVER else {}
+_SESSION = SimpleCookie(_SERVER["HTTP_COOKIE"]) if "HTTP_COOKIE" in _SERVER else SimpleCookie()
 _COOKIE = {k: v.value for k, v in _SESSION.items()}
 _HEADERS = {k: v for k, v in _SERVER.items() if k not in [
     "DOCUMENT_ROOT",
@@ -50,6 +52,7 @@ __response = {
     "status_code": 200,
     "content": b""
 }
+__methods = {}
 PRINTED = {
     "STATUS": False,
     "HEADERS": False,
@@ -69,7 +72,7 @@ def obj_to_bytes(obj):
 
 
 def _print(obj):
-    sys.stdout.buffer.write(obj_to_bytes(obj)+b"\n")
+    sys.stdout.buffer.write(obj_to_bytes(obj))
 
 
 def log(obj):
@@ -92,8 +95,10 @@ def set_header(k, v):
     __headers[k] = v
 
 
-def print(obj):
-    __response["content"] += obj_to_bytes(obj)+b"\n"
+def print(obj, end=b"\n"):
+    __response["content"] += obj_to_bytes(obj)
+    if end:
+        __response["content"] += end
 
 
 def traceback():
@@ -101,32 +106,53 @@ def traceback():
 
 
 def _generate_response():
-    _print("{}: {}".format(
+    _print("{}: {}\n".format(
         "Status",
         __response["status_code"]
     ))
     PRINTED["STATUS"] = True
     for k, v in __headers.items():
-        _print("{}: {}".format(
+        _print("{}: {}\n".format(
             k,
             v
         ))
-    _print("")
+    session = _SESSION.output()
+    if session:
+        session += "\n"
+    _print(session)
+    _print("\n")
     PRINTED["HEADERS"] = True
     _print(__response["content"])
 
 
-def execute(main):
-    def _execute(*args, **kwargs):
-        try:
-            main(*args, **kwargs)
-        except:
-            set_status(500)
-            __response["content"] = traceback()
-        finally:
-            _generate_response()
+def execute(method):
+    def wrapper(main):
+        def _execute():
+            try:
+                main()
+            except:
+                set_status(500)
+                __response["content"] = traceback()
+            finally:
+                try:
+                    _generate_response()
+                except:
+                    try:
+                        set_status(500)
+                    except:
+                        pass
+                    log(_traceback.format_exc())
 
-    return _execute
+        __methods[method] = _execute
+        return _execute
+
+    return wrapper
 
 
+def main():
+    method = _SERVER["REQUEST_METHOD"].lower()
+    if method in __methods:
+        __methods[method]()
+    else:
+        set_status(405)
 
